(*
 * Copyright 2018 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 *)

module mysql;

[external('decc$crtl_init')]
procedure crtl_init; extern;
[external('decc$free')]
procedure free_c_str(ptr : c_str_t); extern;

(*
 * mysql_com.h enum enum_field_types:
 *)
const
   MYSQL_TYPE_DECIMAL = 0;
   MYSQL_TYPE_TINY = 1;
   MYSQL_TYPE_SHORT = 2;
   MYSQL_TYPE_LONG = 3;
   MYSQL_TYPE_FLOAT = 4;
   MYSQL_TYPE_DOUBLE = 5;
   MYSQL_TYPE_NULL = 6;
   MYSQL_TYPE_TIMESTAMP = 7;
   MYSQL_TYPE_LONGLONG = 8;
   MYSQL_TYPE_INT24 = 9;
   MYSQL_TYPE_DATE = 10;
   MYSQL_TYPE_TIME = 11;
   MYSQL_TYPE_DATETIME = 12;
   MYSQL_TYPE_YEAR = 13;
   MYSQL_TYPE_NEWDATE = 14;
   MYSQL_TYPE_VARCHAR = 15;
   MYSQL_TYPE_BIT = 16;
   MYSQL_TYPE_NEWDECIMAL = 246;
   MYSQL_TYPE_ENUM = 247;
   MYSQL_TYPE_SET = 248;
   MYSQL_TYPE_TINY_BLOB = 249;
   MYSQL_TYPE_MEDIUM_BLOB = 250;
   MYSQL_TYPE_LONG_BLOB = 251;
   MYSQL_TYPE_BLOB = 252;
   MYSQL_TYPE_VAR_STRING = 253;
   MYSQL_TYPE_STRING = 254;
   MYSQL_TYPE_GEOMETRY = 255;

type
   (* mysql.h pointer to struct st_mysql: *)
   mysql_ptr = integer_address;
   (* mysql.h pointer to struct st_mysql_stmt: *)
   mysql_stmt_ptr = integer_address;
   (* mysql.h struct st_mysql_bind: *)
   mysql_bind = [aligned(4)] packed record
                    length : integer_address;
                    is_null : integer_address;
                    buffer : integer_address;
                    error : integer_address;
                    row_ptr : integer_address;
                    store_param_func : integer_address;
                    fetch_result : integer_address;
                    skip_result : integer_address;
                    buffer_length : integer;
                    offset : integer;
                    length_value : integer;
                    param_number : integer;
                    pack_length : integer;
                    buffer_type : integer;
                    error_value : boolean;
                    is_unsigned : boolean;
                    long_data_used : boolean;
                    is_null_value : boolean;
                    extension : integer_address;
                end;

(* mysql.h: *)

[external('mysql_init')]
function mysql_init(%IMMED con : mysql_ptr) : mysql_ptr; extern;

[external('mysql_real_connect')]
function mysql_real_connect(%IMMED con : mysql_ptr; %IMMED host, un, pw, db : c_str_t; %IMMED port : integer; %IMMED socket : integer_address; %IMMED flags : integer) : mysql_ptr; extern;

[external('mysql_stmt_init')]
function mysql_stmt_init(%IMMED con : mysql_ptr) : mysql_stmt_ptr; extern;

[external('mysql_stmt_prepare')]
function mysql_stmt_prepare(%IMMED stmt : mysql_stmt_ptr; %IMMED sqlstr : c_str_t; %IMMED sqllen : integer) : integer; extern;

[external('mysql_stmt_bind_param')]
function mysql_stmt_bind_param(%IMMED stmt : mysql_stmt_ptr; %IMMED param : integer_address) : integer; extern;

[external('mysql_stmt_execute')]
function mysql_stmt_execute(%IMMED stmt : mysql_stmt_ptr) : integer; extern;

[external('mysql_stmt_bind_result')]
function mysql_stmt_bind_result(%IMMED stmt : mysql_stmt_ptr; %IMMED param : integer_address) : integer; extern;

[external('mysql_stmt_store_result')]
function mysql_stmt_store_result(%IMMED stmt : mysql_stmt_ptr) : integer; extern;

[external('mysql_stmt_fetch')]
function mysql_stmt_fetch(%IMMED stmt : mysql_stmt_ptr) : integer; extern;

[external('mysql_stmt_free_result')]
function mysql_stmt_free_result(%IMMED stmt : mysql_stmt_ptr) : integer; extern;

[external('mysql_close')]
procedure mysql_close(%IMMED con : mysql_ptr); extern;

[external('mysql_error')]
function mysql_error(%IMMED con : mysql_ptr) : c_str_t; extern;

[external('mysql_stmt_error')]
function mysql_stmt_error(%IMMED stmt : mysql_stmt_ptr) : c_str_t; extern;

[external('mysql_autocommit')]
function mysql_autocommit(%IMMED con : mysql_ptr; %IMMED mode : boolean) : integer; extern;

[external('mysql_commit')]
function mysql_commit(%IMMED con : mysql_ptr) : integer; extern;

[external('mysql_rollback')]
function mysql_rollback(%IMMED con : mysql_ptr) : integer; extern;

[external('mysql_stmt_affected_rows')]
function mysql_stmt_affected_rows(%IMMED stmt : mysql_stmt_ptr) : integer64; extern;

[external('mysql_stmt_field_count')]
function mysql_stmt_field_count(%IMMED stmt : mysql_stmt_ptr) : integer; extern;

[external('mysql_stmt_num_rows')]
function mysql_stmt_num_rows(%IMMED stmt : mysql_stmt_ptr) : integer64; extern;

[external('mysql_stmt_insert_id')]
function mysql_stmt_insert_id(%IMMED stmt : mysql_stmt_ptr) : integer64; extern;

end.
